From 3311e6b20c287a02c69ddee57d21e7b3cfd3de61 Mon Sep 17 00:00:00 2001
From: amit <amitlangote09@gmail.com>
Date: Tue, 31 Oct 2017 16:26:55 +0900
Subject: [PATCH 2/2] Tweak default range partition's constraint a little

When using as a predicate, it's useful for it explicitly say that
the default range partition might contain nulls, because non-default
range partitions can't.
---
 src/backend/catalog/partition.c       | 29 +++++++++++++++++++++++------
 src/test/regress/expected/inherit.out | 12 ++++++++----
 src/test/regress/expected/update.out  |  2 +-
 3 files changed, 32 insertions(+), 11 deletions(-)

diff --git a/src/backend/catalog/partition.c b/src/backend/catalog/partition.c
index ce29ba2eda..d46592c06e 100644
--- a/src/backend/catalog/partition.c
+++ b/src/backend/catalog/partition.c
@@ -2133,12 +2133,29 @@ get_qual_for_range(Relation parent, PartitionBoundSpec *spec,
 
 		if (or_expr_args != NIL)
 		{
-			/* OR all the non-default partition constraints; then negate it */
-			result = lappend(result,
-							 list_length(or_expr_args) > 1
-							 ? makeBoolExpr(OR_EXPR, or_expr_args, -1)
-							 : linitial(or_expr_args));
-			result = list_make1(makeBoolExpr(NOT_EXPR, result, -1));
+			Expr   *other_parts_constr;
+
+			/*
+			 * Combine the constraints obtained for non-default partitions
+			 * using OR.  As requested, each of the OR's args doesn't include
+			 * the NOT NULL test for partition keys (which is to avoid its
+			 * useless repetition).  Add the same now.
+			 */
+			other_parts_constr =
+						makeBoolExpr(AND_EXPR,
+							lappend(get_range_nulltest(key),
+									list_length(or_expr_args) > 1
+										? makeBoolExpr(OR_EXPR, or_expr_args,
+													   -1)
+										: linitial(or_expr_args)),
+									-1);
+
+			/*
+			 * Finally, the default partition contains everything *NOT*
+			 * contained in the non-default partitions.
+			 */
+			result = list_make1(makeBoolExpr(NOT_EXPR,
+										list_make1(other_parts_constr), -1));
 		}
 
 		return result;
diff --git a/src/test/regress/expected/inherit.out b/src/test/regress/expected/inherit.out
index a202caeb25..fac7b62f9c 100644
--- a/src/test/regress/expected/inherit.out
+++ b/src/test/regress/expected/inherit.out
@@ -1861,12 +1861,14 @@ create table mcrparted3 partition of mcrparted for values from (11, 1, 1) to (20
 create table mcrparted4 partition of mcrparted for values from (20, 10, 10) to (20, 20, 20);
 create table mcrparted5 partition of mcrparted for values from (20, 20, 20) to (maxvalue, maxvalue, maxvalue);
 explain (costs off) select * from mcrparted where a = 0;	-- scans mcrparted0, mcrparted_def
-          QUERY PLAN          
-------------------------------
+           QUERY PLAN            
+---------------------------------
  Append
    ->  Seq Scan on mcrparted0
          Filter: (a = 0)
-(3 rows)
+   ->  Seq Scan on mcrparted_def
+         Filter: (a = 0)
+(5 rows)
 
 explain (costs off) select * from mcrparted where a = 10 and abs(b) < 5;	-- scans mcrparted1, mcrparted_def
                  QUERY PLAN                  
@@ -1874,7 +1876,9 @@ explain (costs off) select * from mcrparted where a = 10 and abs(b) < 5;	-- scan
  Append
    ->  Seq Scan on mcrparted1
          Filter: ((a = 10) AND (abs(b) < 5))
-(3 rows)
+   ->  Seq Scan on mcrparted_def
+         Filter: ((a = 10) AND (abs(b) < 5))
+(5 rows)
 
 explain (costs off) select * from mcrparted where a = 10 and abs(b) = 5;	-- scans mcrparted1, mcrparted2, mcrparted_def
                  QUERY PLAN                  
diff --git a/src/test/regress/expected/update.out b/src/test/regress/expected/update.out
index a4fe96112e..b69ceaa75e 100644
--- a/src/test/regress/expected/update.out
+++ b/src/test/regress/expected/update.out
@@ -227,7 +227,7 @@ create table part_def partition of range_parted default;
  a      | text    |           |          |         | extended |              | 
  b      | integer |           |          |         | plain    |              | 
 Partition of: range_parted DEFAULT
-Partition constraint: (NOT (((a = 'a'::text) AND (b >= 1) AND (b < 10)) OR ((a = 'a'::text) AND (b >= 10) AND (b < 20)) OR ((a = 'b'::text) AND (b >= 1) AND (b < 10)) OR ((a = 'b'::text) AND (b >= 10) AND (b < 20))))
+Partition constraint: (NOT ((a IS NOT NULL) AND (b IS NOT NULL) AND (((a = 'a'::text) AND (b >= 1) AND (b < 10)) OR ((a = 'a'::text) AND (b >= 10) AND (b < 20)) OR ((a = 'b'::text) AND (b >= 1) AND (b < 10)) OR ((a = 'b'::text) AND (b >= 10) AND (b < 20)))))
 
 insert into range_parted values ('c', 9);
 -- ok
-- 
2.11.0

